#!/usr/bin/python
# ExcelOutput.py -- Generate an excel output file

import openpyxl
import openpyxl.styles
from openpyxl.worksheet.write_only import WriteOnlyCell

from gaft.mpiutil import master_only

CenterAlign = openpyxl.styles.Alignment(horizontal="center", vertical="center")


# initExcel -- Initialize the output Excel file
@master_only
def initExcel(partnum):
    global CenterAlign

    wb = openpyxl.Workbook(write_only=True)
    ws = wb.create_sheet()
    ws.title = 'Sample Data'
    row = []
    cell = WriteOnlyCell(ws, value='No.')
    cell.alignment = CenterAlign
    row.append(cell)
    for i in range(partnum):
        cell = WriteOnlyCell(ws, value='Period_' + str(i + 1))
        cell.alignment = CenterAlign
        row.append(cell)
        cell = WriteOnlyCell(ws, value='Budget_' + str(i + 1))
        cell.alignment = CenterAlign
        row.append(cell)
    cell = WriteOnlyCell(ws, value='Schedulability(Y/N)')
    cell.alignment = CenterAlign
    row.append(cell)
    cell = WriteOnlyCell(ws, value='Utilization(%)')
    cell.alignment = CenterAlign
    row.append(cell)
    cell = WriteOnlyCell(ws, value=u"Optimal(\u2713)")
    cell.alignment = CenterAlign
    row.append(cell)
    ws.append(row)

    return wb


# outputExcel -- Output a result line to Excel file
@master_only
def outputExcel(wb, partnum, no, prd, budg, issched, realutil, unitscale):
    global CenterAlign

    row = []
    ws = wb.active
    cell = WriteOnlyCell(ws, value=no)
    row.append(cell)
    for i in range(partnum):
        cell = WriteOnlyCell(ws, value=prd[i] / unitscale)
        row.append(cell)
        cell = WriteOnlyCell(ws, value=budg[i] / unitscale)
        row.append(cell)
    cell = WriteOnlyCell(ws, value=issched)
    cell.alignment = CenterAlign
    row.append(cell)
    cell = WriteOnlyCell(ws, value=realutil)
    cell.number_format = '#,##0.00'
    row.append(cell)
    ws.append(row)


# tickOptimum -- Mark the optimal result with a tick in Excel file
@master_only
def tickOptimum(fn, partnum, bestres):
    wb = openpyxl.load_workbook(fn)
    ws = wb.active
    ws.cell(row=bestres + 1, column=2 * partnum + 4).value = u"\u2713"
    wb.save(fn)
