#!/usr/bin/python
# GenQuery.py -- Generate UPPAAL query files

import re


# genVeriStr -- Generate the string of schedulability query (Pr[<=10000](<> error == true) <=...)
def genVeriStr(timebound, prub):
    qstr = 'Pr[<=' + str(timebound) + '](<> error == true) <= ' + str(prub)
    return qstr


# genVeriFile --Generate the verification query file
def genVeriFile(verifile, qrystr):
    qryfile = open(verifile, 'w+')
    qryfile.write(qrystr)
    qryfile.close()


# genPruningStr -- Generate the strings of each partition's schedulability query (Pr[<=10000](<> p1.LackOfSupply...))
def genPruningStr(filecontent, partnum, timebound, prub):
    prefix = 'Pr[<=' + str(timebound) + '](<> '
    qstr = [prefix for i in range(partnum)]
    lno = [-1, -1]

    for i in range(len(filecontent)):
        if re.match(r'// Partitions', filecontent[i]):
            lno[0] = i
        if re.match(r'// Tasks of partition 1', filecontent[i]):
            lno[1] = i
    if lno[0] == -1 or lno[1] == -1:
        print "Error: The UPPAAL model file doesn't have right partition or task declarations!"
        exit(1)

    for i in range(partnum):
        qstr[i] += re.match(r'(.+) = Partition\(.+\);', filecontent[lno[0] + i + 1]).group(1) + '.LackOfSupply'
        for s in re.findall(r'(task\d+) = [a-zA-Z]*Task\(\d+\);', filecontent[lno[1] + 2 * i + 1]):
            qstr[i] += ' or ' + s + '.MissDeadline'
            #qstr[i] += ' or ' + s + '.MissDeadline1' + ' or ' + s + '.MissDeadline2' + ' or ' + s + '.MsgErr'
        qstr[i] += ') <= ' + str(prub)
    return qstr


# pruningFileName -- Get the pruning file name (_queryx.txt)
def pruningFileName(prncmd, pid):
    filestr = re.match(r'verifyta .+\.xml (.+%d.*)', prncmd).group(1)
    return re.sub(r'%d', str(pid), filestr)


# pruningCmd --Get the pruning command for a particular partition (verifyta model.xml _queryx.txt)
def pruningCmd(prncmd, pid):
    return re.sub(r'%d', str(pid), prncmd)


# genPruningFile --Generate the pruning files including all the partitions' schedulability queries
def genPruningFile(prncmd, prnqrys, partnum):
    for i in range(partnum):
        qryfile = open(pruningFileName(prncmd, i), 'w+')
        qryfile.write(prnqrys[i])
        qryfile.close()


# genGaVeriStr -- Generate the strings of schedulability query(Pr[<=10000](<> perror > 0) <=...)
#                 for genetic algorithm
def genGaVeriStr(timebound, prub, partnum):
    qstrs = []
    for i in range(partnum):
        qstrs.append('Pr[<=' + str(timebound) + '](<> perror[' + str(i) + '] > 0) <= ' + str(prub))
    return qstrs


# getQryFn -- Get the query file name from a query command
def getQryFn(cmd):
    return re.match(r'verifyta .+\.xml (.+\.txt)', cmd).group(1)


# getObjQry -- Get the utilization query from an objective command
def getObjQry(objcmd):
    filestr = getQryFn(objcmd)
    ofp = open(filestr, 'r')
    objqry = ofp.read()
    ofp.close()
    return objqry


# genGaVeriStr -- Generate the file of schedulability query for genetic algorithm
def genGaVeriFile(verifile, veriqrys, partnum):
    for i in range(partnum):
        vfp = open('_' + str(i) + '_' + verifile, 'w+')
        vfp.write(veriqrys[i] + '\n')
        vfp.close()
        pass
    #vfp.write(objqry + '\n')


# Update the verification command with the new model file name
def updateVeriCmd(cmd, modelfn):
    return re.sub(r'\s\w+\.xml', ' ' + modelfn, cmd)


# Update the verification command with the new query file name
def gaVeriCmd(cmd, qfn, i):
    return re.sub(r'\s\w+\.txt', ' ' + '_' + str(i) + '_' + qfn, cmd)


# Update the verification command with the new query file name
def genVeriCmd(cmd, modelfn, qfn):
    if modelfn != '':
        cmd = re.sub(r'\s\w+\.xml', ' ' + modelfn, cmd)
    if qfn != '':
        cmd = re.sub(r'\s\w+\.txt', ' ' + qfn, cmd)

    return cmd
