#!/usr/bin/python
# An ARINC 653 partition schedule generator based on periodic partition model


# Import user-defined functions
import LinkedList


# Context switch overhead
cs = 2


# Greatest Common Divisor, GCD
def gcd(a, b):
    if b == 0:
        return a
    else:
        return gcd(b, a % b)


# Least Common Multiple, LCM
def lcm(a, b):
    return (a * b) / gcd(a, b)


# Least Common Multiple, LCM, for more than two numbers
def leastCommonMultiple(vec):
    result = vec[0]
    for i in range(1, len(vec)):
        result = lcm(result, vec[i])

    return result


# Type of node items in linked list
class NodeItem(object):
    """Item type of a node in linked lists"""
    def __init__(self, pno, offset, budget):
        self.pno = pno
        self.offset = offset
        self.budget = budget


# Main procedure for schedule generator
def genSchedule(mainframe, partprd, partbudg, pindex, plist):
    '''
    :param mainframe: major frame of schedule
    :param partprd: period parameter of partition
    :param partbudg: budget parameter of partition
    :param pindex: index of the first partition window for each partition
    :param plist: list of the partition windows of partition schedule
    :return: schedulable partition number
    '''
    NOP = -1
    usage = LinkedList.SinCycLinkedlist()
    usage.add(NodeItem(NOP, mainframe, 0))
    usage.add(NodeItem(NOP, 0, 0))
    pno = 0
    index = 0

    for (prdi, budgi) in zip(partprd, partbudg):
        pindex.append(index)

        j = 0
        offset = 0
        budg = 0
        cur = usage._head
        pre = cur
        while pre.next != usage._head:
            if budg <= 0:
                offset = j * prdi
                budg = budgi

            if offset < cur.item.offset:
                start = pre.item.offset + pre.item.budget
                if offset < start:
                    offset = start
                if cur.item.offset != mainframe:
                    avail = cur.item.offset - offset - 2 * cs
                else:
                    avail = cur.item.offset - offset - cs
                if avail > budg:
                    usage.insert(pre, NodeItem(pno, offset + cs, budg))
                    cur = pre.next
                    plist.append([pno, offset + cs, budg])
                    budg = 0
                    offset += budg
                    index += 1
                if 0 < avail <= budg:
                    usage.insert(pre, NodeItem(pno, offset + cs, avail))
                    pre = pre.next
                    plist.append([pno, offset + cs, avail])
                    budg -= avail
                    offset = cur.item.offset + cur.item.budget
                    index += 1

            if offset >= (j + 1) * prdi and budg > 0:
                return pno
            if offset > (j + 1) * prdi and budg <= 0:
                return pno

            if budg <= 0:
                j += 1

            pre = cur
            cur = cur.next

        pno += 1

    return pno


def genUtil(pno, majorframe, plist):
    '''
    :param pno: number of partitions used for utilization calculation
    :param majorframe: major frame of schedule
    :param plist: list of the partition windows of partition schedule
    :return: utilization value
    '''
    sumofbudg = 0.0

    if pno == 0 or majorframe == 0:
        return sumofbudg

    for pwin in plist:
        if pwin[0] < pno:
            sumofbudg += pwin[2] + cs

    return sumofbudg / majorframe
