#!/usr/bin/python
# ParamSweep.py -- Parameter sweep optimizer

# Import system functions
import time
from sys import argv

# Import user-defined functions
from ConfigInfo import ConfigObj
from ConfigInfo import StatusObj
from ExcelOutput import tickOptimum
from Optim_ga import paramSweep_GA
from gaft.mpiutil import master_only


# finalResult -- Output final result
@master_only
def finalResult(cfg, stat):
    # Close UPPAAL file and save Excel file
    cfg.Fp.close()
    xlsFileName = 'SmcData_' + time.strftime("%Y%m%d_%H%M%S", time.localtime()) + '.xlsx'
    cfg.Workbook.save(filename=xlsFileName)

    # The optimum solution exists
    if stat.BestUtil != 0:
        tickOptimum(xlsFileName, cfg.PartNum, stat.BestRes)
        print "Final result:",
        for i in range(cfg.PartNum):
            if i == 0:
                print "P%d period = %d budget = %d" % (i + 1, stat.BestPrd[i], stat.BestBudg[i]),
            else:
                print "\n              P%d period = %d budget = %d" % (i + 1, stat.BestPrd[i], stat.BestBudg[i]),
        print "\t--->\tUtilization = %.2f" % (stat.BestUtil / 100.0)
    else:
        print "We don't find any solutions!"


# ---Main Function---
if '__main__' == __name__:
    # Read the configuration file
    cfg = ConfigObj('config.ini')
    # Initialize the object of status information
    stat = StatusObj(cfg)

    try:
        # Parameter sweep
        paramSweep_GA(cfg, stat)
    finally:
        # Output final result
        finalResult(cfg, stat)
