#!/usr/bin/env python
# -*- coding: utf-8 -*-

from time import time
from ..plugin_interfaces.analysis import OnTheFlyAnalysis

class FitnessStoreAnalysis(OnTheFlyAnalysis):

    # Analysis interval.
    interval = 1

    # Only analyze in master process?
    master_only = True

    def setup(self, ng, engine):
        # Generation numbers.
        self.ngs = []

        # Best fitness in each generation.
        self.fitness_values = []

        # Best variants.
        self.variants = []

        # Execution time in each generation.
        self.exetimes = []

        # Strategy parameters in each generation.
        self.strategy = []

    def register_step(self, g, population, engine):
        # Collect data.
        best_indv = population.best_indv(engine.fitness)
        best_fit = engine.fitness(best_indv)

        self.ngs.append(g)
        self.variants.append(best_indv.variants)
        self.fitness_values.append(best_fit)
        self.exetimes.append(time())
        self.strategy.append(population.meanstrtgy)

        with open('best_fit.py', 'w') as f:
            f.write('best_fit = [\n')
            for ng, et, x, y, z in zip(self.ngs, self.exetimes, self.variants, self.strategy, self.fitness_values):
                f.write('    ({}, {}, {}, {}, {}),\n'.format(ng, round(et - self.exetimes[0], 1), x, y, z))
            f.write(']\n\n')

    def finalize(self, population, engine):
        with open('best_fit.py', 'w') as f:
            f.write('best_fit = [\n')
            for ng, et, x, y, z in zip(self.ngs, self.exetimes, self.variants, self.strategy, self.fitness_values):
                f.write('    ({}, {}, {}, {}, {}),\n'.format(ng, round(et - self.exetimes[0], 1), x, y, z))
            f.write(']\n\n')

        self.logger.info('Best fitness values are written to best_fit.py')
