#!/usr/bin/env python
# -*- coding: utf-8 -*-

from random import uniform
from random import choice

from ...plugin_interfaces.operators.crossover import GACrossover


class LocalLineRecombination(GACrossover):
    def __init__(self, d=0.25):
        '''
        Crossover operator with local line recombination algorithm,
        see https://en.wikipedia.org/wiki/Crossover_(genetic_algorithm)

        :param d: a scaling factor will be chosen uniformly at random over an interval [-d, 1+d]
        :type d: float in (0.0, 1.0]
        '''

        if d <= 0.0 or d > 1.0:
            raise ValueError('Invalid d value in local line recombination!')

        self.d = d

    def cross(self, father, mother):
        '''
        Cross chromsomes of parent using local line recombination method.
        '''
        # Copy the variants of parents
        vars1 = list(father.variants)
        vars2 = list(mother.variants)

        ai = uniform(-self.d, 1 + self.d)
        for i, (g1, g2) in enumerate(zip(vars1, vars2)):
            # Update factor for each partition
            if i % 2 == 0:
                ai = uniform(-self.d, 1 + self.d)
            vars1[i] = g1 + int(round(ai * (g2 - g1) / father.precisions[i])) * father.precisions[i]
            if vars1[i] < father.ranges[i][0]:
                vars1[i] = father.ranges[i][0]
            if vars1[i] > father.ranges[i][1]:
                vars1[i] = father.ranges[i][1]

        child = father.clone()
        child.init(variants=vars1)
        #child.lasting = max(father.lasting, mother.lasting)

        return [child]
