#!/usr/bin/env python
# -*- coding: utf-8 -*-

from random import random
from random import gauss
from math import ceil
from math import floor

from ...plugin_interfaces.operators.mutation import GAMutation


class DeltaMutation(GAMutation):
    def __init__(self, pm, sigma):
        '''
        Mutation operator with Delta mutation implementation.

        :param pm: The probability of mutation (usually between 0.001 ~ 0.1)
        :type pm: float in (0.0, 1.0]

        :param sigma: the standard deviation of gauss function
        '''
        if pm <= 0.0 or pm > 1.0:
            raise ValueError('Invalid mutation probability')

        self.pm = pm
        self.sigma = sigma

    def mutate(self, individual):
        '''
        Mutate the individual.
        '''
        do_mutation = True if random() <= self.pm else False

        if do_mutation:
            for i, var in enumerate(individual.variants):
                do_delta = True if random() <= self.pm else False
                if do_delta:
                    #var = var + int(round(gauss(0, self.sigma))) * self.step
                    delta = gauss(0, self.sigma)
                    delta = int(ceil(delta / individual.precisions[i])) * individual.precisions[i] if delta > 0\
                        else int(floor(delta / individual.precisions[i])) * individual.precisions[i]
                    var = var + delta
                    if var < individual.ranges[i][0]:
                        var = individual.ranges[i][0]
                    if var > individual.ranges[i][1]:
                        var = individual.ranges[i][1]
                    individual.variants[i] = var

            # Update variants.
            individual.chromsome = individual.encode()

        return individual

