#!/usr/bin/env python
# -*- coding: utf-8 -*-

from random import random
from random import gauss

from ...plugin_interfaces.operators.mutation import GAMutation


class RotatingDeltaMutation(GAMutation):
    def __init__(self, pm, step, sigma):
        '''
        Mutation operator with rotating delta mutation implementation.

        :param pm: The probability of mutation (usually between 0.001 ~ 0.1)
        :type pm: float in (0.0, 1.0]

        :param step: the basic unit of variables in integer

        :param sigma: the standard deviation of gauss function
        '''
        if pm <= 0.0 or pm > 1.0:
            raise ValueError('Invalid mutation probability')
        if not isinstance(step, int):
            raise ValueError('Invalid mutation step')

        self.pm = pm
        self.step = step
        self.sigma = sigma

    def mutate(self, individual):
        '''
        Mutate the individual.
        '''
        do_mutation = True if random() <= self.pm else False

        if do_mutation:
            for i, var in enumerate(individual.variants):
                if i % 2 == 0:
                    x = var
                else:
                    do_delta = True if random() <= self.pm else False
                    if do_delta:
                        y = var
                        delta = gauss(0, self.sigma)

                        deltax = delta * x / (x**2 + y**2)**0.5 + gauss(0, self.step)
                        deltax = int(round(deltax / self.step)) * self.step
                        x = x + deltax

                        deltay = delta * y / (x**2 + y**2)**0.5 + gauss(0, self.step)
                        deltay = int(round(deltay / self.step)) * self.step
                        y = y + deltay

                        if x < individual.ranges[i - 1][0]:
                            x = individual.ranges[i - 1][0]
                        if x > individual.ranges[i - 1][1]:
                            x = individual.ranges[i - 1][1]
                        individual.variants[i - 1] = x

                        if y < individual.ranges[i][0]:
                            y = individual.ranges[i][0]
                        if y > individual.ranges[i][1]:
                            y = individual.ranges[i][1]
                        individual.variants[i] = y

            # Update variants.
            individual.chromsome = individual.encode()

        return individual

