#!/usr/bin/env python
# -*- coding: utf-8 -*-

''' Logarithmic Ranking Selection implementation. '''

from random import random
#from itertools import accumulate
from bisect import bisect_right
from math import log

from ...plugin_interfaces.operators.selection import GASelection


class LogarithmicRankingSelection(GASelection):
    def __init__(self, base=10):
        '''
        Selection operator using Logarithmic Ranking selection method.

        :param base: The base of logarithm
        :type base: int
        '''

        self.base = base

    def select(self, population, fitness):
        '''
        Select a pair of parent individuals using exponential ranking method.
        '''
        # The size of population is the minimum base of logarithm.
        NP = len(population)
        if self.base > NP:
            NP = self.base
        if NP < 2:
            raise ValueError('The base of logarithm must be greater than 2 !')

        # Add rank to all individuals in population.
        sorted_indvs = sorted(population.individuals, key=fitness)

        # NOTE: Here the rank i belongs to {1, ..., N}
        p = lambda i: log(i + 1, NP)
        probabilities = [p(i) for i in range(0, NP)]

        # Normalize probabilities.
        psum = sum(probabilities)
        #wheel = list(accumulate([p/psum for p in probabilities]))
        wheel = []
        accvl = 0
        for k in [p / psum for p in probabilities]:
            accvl += k
            wheel.append(accvl)
            pass

        # Select parents.
        father_idx = bisect_right(wheel, random())
        father = sorted_indvs[father_idx]
        mother_idx = (father_idx + 1) % len(wheel)
        mother = sorted_indvs[mother_idx]

        return father, mother
